/**
 * Copyright Zendesk, Inc.
 *
 * Use of this source code is governed under the Apache License, Version 2.0
 * found at http://www.apache.org/licenses/LICENSE-2.0.
 */

import React, { useState, useRef, useMemo, useCallback, useLayoutEffect, useEffect } from 'react';
import { useField } from '@zendeskgarden/container-field';
import { KEYS, useId, composeEventHandlers } from '@zendeskgarden/container-utilities';
import { useCombobox as useCombobox$1 } from 'downshift';
import PropTypes from 'prop-types';

const typeMap = {
  [useCombobox$1.stateChangeTypes.FunctionCloseMenu]: 'fn:setExpansion',
  [useCombobox$1.stateChangeTypes.FunctionOpenMenu]: 'fn:setExpansion',
  [useCombobox$1.stateChangeTypes.FunctionToggleMenu]: 'fn:setExpansion',
  [useCombobox$1.stateChangeTypes.FunctionReset]: 'fn:reset',
  [useCombobox$1.stateChangeTypes.FunctionSelectItem]: 'fn:setSelectionValue',
  [useCombobox$1.stateChangeTypes.FunctionSetHighlightedIndex]: 'fn:setActiveIndex',
  [useCombobox$1.stateChangeTypes.FunctionSetInputValue]: 'fn:setInputValue',
  [useCombobox$1.stateChangeTypes.InputBlur]: 'input:blur',
  [useCombobox$1.stateChangeTypes.InputChange]: 'input:change',
  [useCombobox$1.stateChangeTypes.InputClick]: 'input:click',
  [useCombobox$1.stateChangeTypes.InputKeyDownArrowDown]: `input:keyDown:${KEYS.DOWN}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownArrowUp]: `input:keyDown:${KEYS.UP}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownEnd]: `input:keyDown:${KEYS.END}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownEnter]: `input:keyDown:${KEYS.ENTER}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownEscape]: `input:keyDown:${KEYS.ESCAPE}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownHome]: `input:keyDown:${KEYS.HOME}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownPageDown]: `input:keyDown:${KEYS.PAGE_DOWN}`,
  [useCombobox$1.stateChangeTypes.InputKeyDownPageUp]: `input:keyDown:${KEYS.PAGE_UP}`,
  [useCombobox$1.stateChangeTypes.ItemClick]: 'option:click',
  [useCombobox$1.stateChangeTypes.ItemMouseMove]: 'option:mouseMove',
  [useCombobox$1.stateChangeTypes.MenuMouseLeave]: 'listbox:mouseLeave',
  [useCombobox$1.stateChangeTypes.ToggleButtonClick]: 'toggle:click'
};
const toType = downshiftType => {
  return typeMap[downshiftType] || downshiftType;
};
const toLabel = (labels, value) => {
  if (value === undefined) {
    return '';
  }
  return labels[value];
};

const useCombobox = _ref => {
  let {
    idPrefix,
    triggerRef,
    inputRef,
    listboxRef,
    isAutocomplete,
    isMultiselectable,
    isEditable = true,
    disabled,
    hasHint,
    hasMessage,
    options = [],
    inputValue,
    selectionValue,
    isExpanded,
    defaultExpanded,
    initialExpanded,
    activeIndex,
    defaultActiveIndex,
    initialActiveIndex,
    onChange = () => undefined,
    environment
  } = _ref;
  const win = environment || window;
  const [triggerContainsInput, setTriggerContainsInput] = useState();
  const [downshiftInputValue, setDownshiftInputValue] = useState(inputValue);
  const [matchValue, setMatchValue] = useState('');
  const useInputValueRef = useRef(true);
  const matchTimeoutRef = useRef();
  const previousStateRef = useRef();
  const prefix = useId(idPrefix);
  const idRef = useRef({
    label: `${prefix}--label`,
    hint: `${prefix}--hint`,
    trigger: `${prefix}--trigger`,
    input: `${prefix}--input`,
    listbox: `${prefix}--listbox`,
    message: `${prefix}--message`,
    getOptionId: (index, isDisabled, isHidden) => `${prefix}--option${isDisabled ? '-disabled' : ''}${isHidden ? '-hidden' : ''}-${index}`
  });
  const labels = useMemo(() => ({}), []);
  const selectedValues = useMemo(() => [], []);
  const disabledValues = useMemo(() => [], []);
  const hiddenValues = useMemo(() => [], []);
  const values = useMemo(() => {
    const retVal = [];
    const setValues = option => {
      if (option.disabled || option.hidden) {
        if (option.disabled && !disabledValues.includes(option.value)) {
          disabledValues.push(option.value);
        }
        if (option.hidden && !hiddenValues.includes(option.value)) {
          hiddenValues.push(option.value);
        }
      } else {
        retVal.push(option.value);
        const disabledIndex = disabledValues.indexOf(option.value);
        if (disabledIndex !== -1) {
          disabledValues.splice(disabledIndex, 1);
        }
        const hiddenIndex = hiddenValues.indexOf(option.value);
        if (hiddenIndex !== -1) {
          hiddenValues.splice(hiddenIndex, 1);
        }
      }
      if (option.selected && !selectedValues.includes(option.value)) {
        selectedValues.push(option.value);
      }
      labels[option.value] = option.label || option.value;
    };
    options.forEach(option => {
      if ('options' in option) {
        option.options.forEach(setValues);
      } else {
        setValues(option);
      }
    });
    return retVal;
  }, [options, disabledValues, hiddenValues, selectedValues, labels]);
  const initialSelectionValue = isMultiselectable ? selectedValues : selectedValues[0];
  const initialInputValue = isMultiselectable ? '' : toLabel(labels, initialSelectionValue);
  const _defaultActiveIndex = useMemo(() => {
    if (defaultActiveIndex === undefined) {
      return isAutocomplete && isEditable ? 0 : undefined;
    }
    return defaultActiveIndex;
  }, [defaultActiveIndex, isAutocomplete, isEditable]);
  if (useInputValueRef.current && inputValue !== downshiftInputValue) {
    setDownshiftInputValue(inputValue);
  } else {
    useInputValueRef.current = true;
  }
  if (selectionValue === undefined || selectionValue === null) {
    if (!isMultiselectable && selectedValues.length > 1) {
      throw new Error('Error: expected useCombobox `options` to have no more than one selected.');
    }
  }
  if (selectionValue !== undefined && selectionValue !== null) {
    if (isMultiselectable && !Array.isArray(selectionValue)) {
      throw new Error('Error: expected multiselectable useCombobox `selectionValue` to be an array.');
    } else if (!isMultiselectable && Array.isArray(selectionValue)) {
      throw new Error('Error: expected useCombobox `selectionValue` not to be an array.');
    }
  }
  const handleDownshiftStateChange = useCallback(_ref2 => {
    let {
      type,
      isOpen,
      selectedItem,
      inputValue: _inputValue,
      highlightedIndex
    } = _ref2;
    return onChange({
      type: toType(type),
      ...(isOpen !== undefined && {
        isExpanded: isOpen
      }),
      ...(selectedItem !== undefined && {
        selectionValue: selectedItem
      }),
      ...(_inputValue !== undefined && {
        inputValue: _inputValue
      }),
      ...(highlightedIndex !== undefined && {
        activeIndex: highlightedIndex
      })
    });
  }, [onChange]);
  const stateReducer = (state, _ref3) => {
    let {
      type,
      changes,
      altKey
    } = _ref3;
    switch (type) {
      case useCombobox$1.stateChangeTypes.ControlledPropUpdatedSelectedItem:
        return state;
      case useCombobox$1.stateChangeTypes.FunctionSetHighlightedIndex:
        if (previousStateRef.current?.altKey) {
          changes.highlightedIndex = -1;
        }
        break;
      case useCombobox$1.stateChangeTypes.FunctionCloseMenu:
      case useCombobox$1.stateChangeTypes.InputBlur:
        return {
          ...state,
          isOpen: type === useCombobox$1.stateChangeTypes.InputBlur && triggerContainsInput && isMultiselectable && state.isOpen || false
        };
      case useCombobox$1.stateChangeTypes.InputClick:
        if (!isAutocomplete) {
          changes.isOpen = state.isOpen;
        }
        break;
      case useCombobox$1.stateChangeTypes.InputKeyDownArrowDown:
      case useCombobox$1.stateChangeTypes.FunctionOpenMenu:
        if (state.isOpen !== changes.isOpen && !altKey) {
          changes.highlightedIndex = 0;
        }
        break;
      case useCombobox$1.stateChangeTypes.InputKeyDownArrowUp:
        if (state.isOpen !== changes.isOpen) {
          changes.highlightedIndex = values.length - 1;
        }
        break;
      case useCombobox$1.stateChangeTypes.InputKeyDownEnter:
      case useCombobox$1.stateChangeTypes.FunctionSelectItem:
      case useCombobox$1.stateChangeTypes.ItemClick:
        changes.highlightedIndex = state.highlightedIndex;
        if (isMultiselectable) {
          changes.isOpen = state.isOpen;
          changes.inputValue = '';
        }
        break;
      case useCombobox$1.stateChangeTypes.InputKeyDownEscape:
        return {
          ...state,
          isOpen: false
        };
      case useCombobox$1.stateChangeTypes.InputKeyDownPageDown:
      case useCombobox$1.stateChangeTypes.InputKeyDownPageUp:
        return state;
    }
    if (isMultiselectable && state.selectedItem !== changes.selectedItem) {
      if (state.selectedItem !== undefined && state.selectedItem !== null && changes.selectedItem !== undefined && changes.selectedItem !== null) {
        if (state.selectedItem.includes(changes.selectedItem)) {
          changes.selectedItem = state.selectedItem.filter(value => value !== changes.selectedItem);
        } else {
          changes.selectedItem = [...state.selectedItem, changes.selectedItem];
        }
      } else if (changes.selectedItem !== undefined && changes.selectedItem !== null) {
        changes.selectedItem = [changes.selectedItem];
      } else {
        changes.selectedItem = [];
      }
    }
    previousStateRef.current = {
      type,
      altKey,
      ...state
    };
    return changes;
  };
  const transformValue = value => value ? toLabel(labels, value) : '';
  const {
    selectedItem: _selectionValue,
    isOpen: _isExpanded,
    highlightedIndex: _activeIndex,
    inputValue: _inputValue,
    getToggleButtonProps: getDownshiftTriggerProps,
    getInputProps: getDownshiftInputProps,
    getMenuProps: getDownshiftListboxProps,
    getItemProps: getDownshiftOptionProps,
    closeMenu,
    openMenu,
    setHighlightedIndex,
    selectItem
  } = useCombobox$1({
    toggleButtonId: idRef.current.trigger,
    menuId: idRef.current.listbox,
    getItemId: idRef.current.getOptionId,
    items: values,
    inputValue: downshiftInputValue,
    initialInputValue,
    itemToString: transformValue ,
    selectedItem: selectionValue,
    initialSelectedItem: initialSelectionValue,
    isOpen: isExpanded,
    defaultIsOpen: defaultExpanded,
    initialIsOpen: initialExpanded,
    highlightedIndex: activeIndex,
    defaultHighlightedIndex: _defaultActiveIndex,
    initialHighlightedIndex: initialActiveIndex,
    onStateChange: handleDownshiftStateChange,
    stateReducer,
    environment: win
  });
  const closeListbox = useCallback(() => {
    closeMenu();
    onChange({
      type: toType(useCombobox$1.stateChangeTypes.FunctionCloseMenu),
      isExpanded: false
    });
  }, [closeMenu, onChange]);
  const openListbox = useCallback(() => {
    openMenu();
    onChange({
      type: toType(useCombobox$1.stateChangeTypes.FunctionOpenMenu),
      isExpanded: true
    });
  }, [openMenu, onChange]);
  const setActiveIndex = useCallback(index => {
    setHighlightedIndex(index);
    onChange({
      type: toType(useCombobox$1.stateChangeTypes.FunctionSetHighlightedIndex),
      activeIndex: index
    });
  }, [onChange, setHighlightedIndex]);
  const setDownshiftSelection = useCallback(value => {
    selectItem(value);
    onChange({
      type: toType(useCombobox$1.stateChangeTypes.FunctionSelectItem),
      selectionValue: value
    });
  }, [onChange, selectItem]);
  const {
    getLabelProps: getFieldLabelProps,
    getHintProps: getFieldHintProps,
    getInputProps: getFieldInputProps,
    getMessageProps: getFieldMessageProps
  } = useField({
    hasHint,
    hasMessage
  });
  useLayoutEffect(() => {
    if ((isAutocomplete || !isEditable) && _isExpanded && !previousStateRef.current?.isOpen && _selectionValue && !matchValue) {
      const value = Array.isArray(_selectionValue) ? _selectionValue[_selectionValue.length - 1
      ] : _selectionValue;
      const index = values.findIndex(current => current === value);
      if (index !== -1) {
        setActiveIndex(index);
      } else if (_defaultActiveIndex !== undefined) {
        setActiveIndex(_defaultActiveIndex);
      }
    }
  }, [
  isAutocomplete, isEditable, _isExpanded, _selectionValue, _inputValue, values, _defaultActiveIndex, setActiveIndex]);
  useEffect(
  () => setTriggerContainsInput(triggerRef.current?.contains(inputRef.current)), [triggerRef, inputRef]);
  useEffect(() => {
    clearTimeout(matchTimeoutRef.current);
    matchTimeoutRef.current = window.setTimeout(() => setMatchValue(''), 500);
    return () => clearTimeout(matchTimeoutRef.current);
  }, [matchValue]);
  useEffect(() => {
    if (previousStateRef.current?.type === useCombobox$1.stateChangeTypes.FunctionSelectItem) {
      if (isEditable) {
        inputRef.current?.focus();
      } else {
        triggerRef.current?.focus();
      }
      previousStateRef.current = {
        ...previousStateRef.current,
        type: useCombobox$1.stateChangeTypes.InputClick
      };
    }
  });
  useEffect(() => {
    if (isEditable && inputRef.current === win.document.activeElement) {
      inputRef.current?.scrollIntoView && inputRef.current?.scrollIntoView({
        block: 'nearest'
      });
    }
  }, [inputRef, isEditable, win.document.activeElement]);
  const getTriggerProps = useCallback(function (_temp) {
    let {
      onBlur,
      onClick,
      onKeyDown,
      ...other
    } = _temp === void 0 ? {} : _temp;
    const triggerProps = getDownshiftTriggerProps({
      'data-garden-container-id': 'containers.combobox',
      'data-garden-container-version': '2.0.4',
      onBlur,
      onClick,
      onKeyDown,
      ref: triggerRef,
      disabled,
      ...other
    });
    const handleBlur = event => {
      if (event.relatedTarget === null || !event.currentTarget?.contains(event.relatedTarget)) {
        closeListbox();
      }
    };
    if (isEditable && triggerContainsInput) {
      const handleClick = event => {
        if (disabled) {
          event.preventDefault();
        } else if (isAutocomplete) {
          triggerProps.onClick && triggerProps.onClick(event);
        } else {
          inputRef.current?.focus();
        }
      };
      return {
        ...triggerProps,
        onBlur: composeEventHandlers(onBlur, handleBlur),
        onClick: composeEventHandlers(onClick, handleClick),
        'aria-controls': isAutocomplete ? triggerProps['aria-controls'] : undefined,
        'aria-expanded': undefined,
        'aria-disabled': disabled || undefined,
        disabled: undefined
      };
    } else if (!isEditable) {
      const {
        'aria-activedescendant': ariaActiveDescendant,
        onKeyDown: onDownshiftKeyDown
      } = getDownshiftInputProps({}, {
        suppressRefError: true
      });
      const handleKeyDown = event => {
        event.stopPropagation();
        if (!_isExpanded && (event.key === KEYS.SPACE || event.key === KEYS.ENTER)) {
          event.preventDefault();
          openListbox();
        } else if (_isExpanded && !matchValue && (event.key === KEYS.SPACE || event.key === KEYS.ENTER)) {
          event.preventDefault();
          if (_activeIndex !== -1) {
            setDownshiftSelection(values[_activeIndex]);
          }
          if (!isMultiselectable) {
            closeListbox();
          }
        } else if (/^(?:\S| ){1}$/u.test(event.key)) {
          const _matchValue = `${matchValue}${event.key}`;
          setMatchValue(_matchValue);
          let offset = 0;
          if (_isExpanded) {
            if (_activeIndex !== -1) {
              offset = _matchValue.length === 1 ? _activeIndex + 1 : _activeIndex;
            }
          } else {
            openListbox();
            const offsetValue = Array.isArray(_selectionValue) ? _selectionValue[_selectionValue.length - 1
            ] : _selectionValue;
            if (offsetValue !== null) {
              offset = values.findIndex(current => current === offsetValue);
            }
          }
          for (let index = 0; index < values.length; index++) {
            const valueIndex = (index + offset) % values.length;
            const value = values[valueIndex];
            if (toLabel(labels, value).toLowerCase().startsWith(_matchValue.toLowerCase())) {
              setActiveIndex(valueIndex);
              break;
            }
          }
        }
      };
      return {
        ...triggerProps,
        'aria-activedescendant': ariaActiveDescendant,
        'aria-haspopup': 'listbox',
        'aria-labelledby': idRef.current.label,
        'aria-disabled': disabled || undefined,
        disabled: undefined,
        role: 'combobox',
        onBlur: composeEventHandlers(onBlur, handleBlur),
        onKeyDown: composeEventHandlers(onKeyDown, onDownshiftKeyDown, handleKeyDown),
        tabIndex: disabled ? -1 : 0
      };
    }
    return triggerProps;
  }, [getDownshiftTriggerProps, getDownshiftInputProps, triggerRef, disabled, _selectionValue, _isExpanded, _activeIndex, closeListbox, openListbox, setActiveIndex, setDownshiftSelection, matchValue, values, labels, triggerContainsInput, isAutocomplete, isEditable, isMultiselectable, inputRef]);
  const getLabelProps = useCallback(function (_temp2) {
    let {
      onClick,
      ...other
    } = _temp2 === void 0 ? {} : _temp2;
    const {
      htmlFor,
      ...labelProps
    } = getFieldLabelProps({
      id: idRef.current.label,
      htmlFor: idRef.current.input,
      ...other
    });
    const handleClick = () => !isEditable && triggerRef.current?.focus();
    return {
      ...labelProps,
      onClick: composeEventHandlers(onClick, handleClick),
      htmlFor: isEditable ? htmlFor : undefined
    };
  }, [getFieldLabelProps, isEditable, triggerRef]);
  const getHintProps = useCallback(props => getFieldHintProps({
    id: idRef.current.hint,
    ...props
  }), [getFieldHintProps]);
  const getInputProps = useCallback(function (_temp3) {
    let {
      role = isEditable ? 'combobox' : null,
      onChange: _onChange,
      onClick,
      onFocus,
      ...other
    } = _temp3 === void 0 ? {} : _temp3;
    const inputProps = {
      'data-garden-container-id': 'containers.combobox.input',
      'data-garden-container-version': '2.0.4',
      ref: inputRef,
      role: role === null ? undefined : role,
      onChange: _onChange,
      onClick,
      onFocus
    };
    if (isEditable) {
      const handleChange = event => {
        if (inputValue !== undefined) {
          setDownshiftInputValue(event.target.value);
          useInputValueRef.current = false;
          if (event.nativeEvent.isComposing) {
            handleDownshiftStateChange({
              type: useCombobox$1.stateChangeTypes.InputChange,
              inputValue: event.target.value
            });
          }
        }
      };
      const handleClick = event => event.target instanceof Element && triggerRef.current?.contains(event.target) && event.stopPropagation();
      const describedBy = [];
      if (hasHint) {
        describedBy.push(idRef.current.hint);
      }
      if (hasMessage) {
        describedBy.push(idRef.current.message);
      }
      return getDownshiftInputProps({
        ...inputProps,
        disabled,
        role,
        'aria-autocomplete': isAutocomplete ? 'list' : undefined,
        onChange: composeEventHandlers(_onChange, handleChange),
        onClick: composeEventHandlers(onClick, handleClick),
        ...getFieldInputProps({
          id: idRef.current.input,
          'aria-labelledby': idRef.current.label,
          'aria-describedby': describedBy.length > 0 ? describedBy.join(' ') : undefined
        }),
        ...other
      });
    }
    const downshiftInputProps = getDownshiftInputProps({
      ...inputProps,
      disabled: true,
      'aria-autocomplete': undefined,
      'aria-activedescendant': undefined,
      'aria-controls': undefined,
      'aria-expanded': undefined,
      'aria-hidden': true,
      'aria-labelledby': undefined
    });
    const handleFocus = () => {
      if (!isEditable) {
        triggerRef.current?.focus();
      }
    };
    return {
      ...downshiftInputProps,
      disabled,
      readOnly: true,
      tabIndex: -1,
      onFocus: composeEventHandlers(onFocus, handleFocus),
      ...other
    };
  }, [getDownshiftInputProps, getFieldInputProps, handleDownshiftStateChange, hasHint, hasMessage, inputValue, inputRef, triggerRef, disabled, isAutocomplete, isEditable]);
  const getTagProps = useCallback(_ref4 => {
    let {
      option,
      onClick,
      onKeyDown,
      ...other
    } = _ref4;
    const handleClick = event => event.target instanceof Element && triggerRef.current?.contains(event.target) && event.stopPropagation();
    const handleKeyDown = event => {
      if (event.key === KEYS.BACKSPACE || event.key === KEYS.DELETE) {
        setDownshiftSelection(option.value);
      } else {
        const triggerContainsTag = event.target instanceof Element && triggerRef.current?.contains(event.target);
        if (triggerContainsTag && !isEditable) {
          event.stopPropagation();
        }
        if (triggerContainsTag && (event.key === KEYS.DOWN || event.key === KEYS.UP || event.key === KEYS.ESCAPE || !isEditable && (event.key === KEYS.ENTER || event.key === KEYS.SPACE))) {
          const inputProps = getDownshiftInputProps();
          if (isEditable) {
            inputRef.current?.focus();
          } else {
            event.preventDefault();
            triggerRef.current?.focus();
          }
          inputProps.onKeyDown && inputProps.onKeyDown(event);
        }
      }
    };
    return {
      'data-garden-container-id': 'containers.combobox.tag',
      'data-garden-container-version': '2.0.4',
      onClick: composeEventHandlers(onClick, handleClick),
      onKeyDown: composeEventHandlers(onKeyDown, handleKeyDown),
      ...other
    };
  }, [triggerRef, setDownshiftSelection, getDownshiftInputProps, isEditable, inputRef]);
  const getListboxProps = useCallback(_ref5 => {
    let {
      role = 'listbox',
      ...other
    } = _ref5;
    return getDownshiftListboxProps({
      'data-garden-container-id': 'containers.combobox.listbox',
      'data-garden-container-version': '2.0.4',
      ref: listboxRef,
      role,
      'aria-multiselectable': isMultiselectable ? true : undefined,
      ...other
    });
  }, [getDownshiftListboxProps, listboxRef, isMultiselectable]);
  const getOptGroupProps = useCallback(_ref6 => {
    let {
      role = 'group',
      ...other
    } = _ref6;
    return {
      'data-garden-container-id': 'containers.combobox.optgroup',
      'data-garden-container-version': '2.0.4',
      role: role === null ? undefined : role,
      ...other
    };
  }, []);
  const getOptionProps = useCallback(function (_temp4) {
    let {
      role = 'option',
      option,
      onMouseDown,
      ...other
    } = _temp4 === void 0 ? {} : _temp4;
    const optionProps = {
      'data-garden-container-id': 'containers.combobox.option',
      'data-garden-container-version': '2.0.4',
      role,
      onMouseDown,
      ...other
    };
    let ariaSelected = false;
    if (option?.value !== undefined) {
      ariaSelected = Array.isArray(_selectionValue) ? _selectionValue?.includes(option?.value) : _selectionValue === option?.value;
    }
    if (option?.hidden) {
      return {
        'aria-disabled': option.disabled ? true : undefined,
        'aria-hidden': true,
        'aria-selected': ariaSelected,
        id: option ? idRef.current.getOptionId(hiddenValues.indexOf(option.value), option.disabled, option.hidden) : undefined,
        ...optionProps
      };
    }
    if (option === undefined || option.disabled) {
      const handleMouseDown = event => event.preventDefault();
      return {
        'aria-disabled': true,
        'aria-selected': ariaSelected,
        id: option ? idRef.current.getOptionId(disabledValues.indexOf(option.value), option.disabled, option.hidden) : undefined,
        ...optionProps,
        onMouseDown: composeEventHandlers(onMouseDown, handleMouseDown)
      };
    }
    return getDownshiftOptionProps({
      item: option.value,
      index: values.indexOf(option.value),
      'aria-disabled': undefined,
      'aria-hidden': undefined,
      'aria-selected': ariaSelected,
      ...optionProps
    });
  }, [getDownshiftOptionProps, disabledValues, hiddenValues, values, _selectionValue]);
  const getMessageProps = useCallback(props => getFieldMessageProps({
    id: idRef.current.message,
    ...props
  }), [getFieldMessageProps]);
  const removeSelection = useCallback(value => {
    if (value === undefined) {
      setDownshiftSelection(null);
    } else {
      const removeValue = typeof value === 'object' && 'value' in value ? value.value : value;
      if (Array.isArray(_selectionValue) && _selectionValue.includes(removeValue)) {
        setDownshiftSelection(removeValue);
      } else if (_selectionValue === removeValue) {
        setDownshiftSelection(null);
      } else if (process.env.NODE_ENV === 'development' || process.env.NODE_ENV === 'test') {
        console.warn(`Warning: useCombobox \`selection\` does not contain '${removeValue}' for removal.`);
      }
    }
  }, [_selectionValue, setDownshiftSelection]);
  const selection = useMemo(() => {
    if (Array.isArray(_selectionValue)) {
      return _selectionValue.map(value => ({
        value,
        label: labels[value],
        disabled: disabledValues.includes(value),
        hidden: hiddenValues.includes(value)
      }));
    } else if (_selectionValue) {
      return {
        value: _selectionValue,
        label: toLabel(labels, _selectionValue),
        disabled: disabledValues.includes(_selectionValue),
        hidden: hiddenValues.includes(_selectionValue)
      };
    }
    return null;
  }, [_selectionValue, disabledValues, hiddenValues, labels]);
  return useMemo(() => ({
    getLabelProps,
    getHintProps,
    getTriggerProps,
    getInputProps,
    getTagProps,
    getListboxProps,
    getOptGroupProps,
    getOptionProps,
    getMessageProps,
    selection,
    isExpanded: _isExpanded,
    activeValue: values[_activeIndex],
    inputValue: _inputValue,
    removeSelection
  }), [values, selection, _isExpanded, _activeIndex, _inputValue, getLabelProps, getHintProps, getTriggerProps, getInputProps, getTagProps, getListboxProps, getOptGroupProps, getOptionProps, getMessageProps, removeSelection]);
};

const ComboboxContainer = props => {
  const {
    children,
    render = children,
    ...options
  } = props;
  return React.createElement(React.Fragment, null, render(useCombobox(options)));
};
ComboboxContainer.propTypes = {
  children: PropTypes.func,
  render: PropTypes.func,
  idPrefix: PropTypes.string,
  triggerRef: PropTypes.any.isRequired,
  inputRef: PropTypes.any.isRequired,
  listboxRef: PropTypes.any.isRequired,
  isAutocomplete: PropTypes.bool,
  isMultiselectable: PropTypes.bool,
  isEditable: PropTypes.bool,
  disabled: PropTypes.bool,
  hasHint: PropTypes.bool,
  hasMessage: PropTypes.bool,
  options: PropTypes.arrayOf(PropTypes.any).isRequired,
  inputValue: PropTypes.string,
  selectionValue: PropTypes.any,
  isExpanded: PropTypes.bool,
  defaultExpanded: PropTypes.bool,
  initialExpanded: PropTypes.bool,
  activeIndex: PropTypes.number,
  defaultActiveIndex: PropTypes.number,
  initialActiveIndex: PropTypes.number,
  onChange: PropTypes.func,
  environment: PropTypes.any
};
ComboboxContainer.defaultProps = {
  isEditable: true
};

export { ComboboxContainer, useCombobox };
